#include "shared.hpp"

#include <spdlog/async.h>
#include <spdlog/cfg/env.h>
#include <spdlog/logger.h>
#include <spdlog/sinks/basic_file_sink.h>
#include <spdlog/sinks/rotating_file_sink.h>
#include <spdlog/sinks/stdout_color_sinks.h>
#include <spdlog/spdlog.h>

#include <memory>
#include <mutex>
#include <sole.hpp>

namespace tsdb_shared {

std::once_flag tsdb_env_init_flag;

void init_tsdb_env_inner() {
    // Initialize the TSDB environment

    auto env_is_set = [](const char* name) {
        auto value = std::getenv(name);
        return value && value[0] != '\0';
    };

    spdlog::cfg::load_env_levels();
    auto use_file_logging = env_is_set("TSDB_USE_FILE_LOGGING");
    auto log_path         = std::getenv("TSDB_LOG_PATH") ?: "tsdb.log";
    spdlog::init_thread_pool(8192, 1);
    std::shared_ptr<spdlog::logger> logger;
    if (use_file_logging) {
        logger = spdlog::basic_logger_mt<spdlog::async_factory>("tsdb", log_path);
    } else {
        logger = spdlog::stdout_color_mt<spdlog::async_factory>("tsdb");
    }
    spdlog::set_default_logger(logger);
}

void init_tsdb_env() {
    std::call_once(tsdb_env_init_flag, init_tsdb_env_inner);
}

std::string random_uuid() {
    return sole::uuid4().str();
}

}  // namespace tsdb_shared
